import pandas as pd
import os
import re
import logging
import PyLibrary.funciones_comunes as fc

def preprocesar_importacion(tipo_PLC, codigo_columna_maestra, df_importacion, logger):
    filas_validas = []
    
    for index, fila in df_importacion.iterrows():
        clave_original = fila[codigo_columna_maestra]
        
        # Verificar si la celda de la columna maestra está vacía o solo contiene espacios en blanco
        if pd.isna(clave_original) or str(clave_original).strip() == '':
            logger.info(f"Fila {index}: Ignorada - Celda de columna maestra vacía o solo con espacios en blanco")
            continue
        
        # Aplicar unescape solo a las filas con valor no vacío en la columna maestra
        nueva_fila = fila.copy()
        for col in nueva_fila.index:
            nueva_fila[col] = fc.limpiar_caracteres_especiales(str(nueva_fila[col]))
        
        clave_sustituida = fc.compactar_celda_clave(tipo_PLC, str(nueva_fila[codigo_columna_maestra]))
        nueva_fila[codigo_columna_maestra] = clave_sustituida
        
        for columna in df_importacion.columns:
            if columna != codigo_columna_maestra and fc.es_columna_tipo_xxYY(columna):
                nueva_fila[columna] = fc.compactar_celda_traducida(tipo_PLC, nueva_fila[columna])
        
        filas_validas.append(nueva_fila)
        
        logger.info(f"Fila {index}: Aceptada - Clave '{clave_sustituida}' procesada")

    df_valido = pd.DataFrame(filas_validas, columns=df_importacion.columns)
    return df_valido

def importar(tipo_PLC, codigo_columna_maestra, archivo_maestro, archivo_importacion):
    logger = fc.configurar_logger()
    logger.info("    ....    ")
    logger.info(f"Iniciando importacion en {archivo_maestro} desde el archivo {archivo_importacion}")
    
    if not os.path.exists(archivo_maestro):
        df_maestro = pd.DataFrame(columns=[codigo_columna_maestra])
    else:
        df_maestro = fc.read_dataframe_with_cleanup_retries(archivo_maestro)
        # Aplicar unescape solo a las filas con valor no vacío en la columna maestra

    df_importacion = pd.read_excel(archivo_importacion)

    df_importacion = preprocesar_importacion(tipo_PLC, codigo_columna_maestra, df_importacion, logger)

    claves_maestro = set(df_maestro[codigo_columna_maestra].dropna().astype(str))

    filas_a_agregar = []

    for index, fila in df_importacion.iterrows():
        clave = str(fila[codigo_columna_maestra])
        
        if clave in claves_maestro:
            logger.info(f"Fila {index}: Ignorada - Clave '{clave}' ya existe en el archivo maestro")
            continue
        
        claves_maestro.add(clave)
        
        fila_filtrada = {
            col: fila[col]
            for col in fila.index
            if col == codigo_columna_maestra or fc.es_columna_tipo_xxYY(col)
        }
        filas_a_agregar.append(fila_filtrada)

    if not filas_a_agregar:
        logger.info("No hay nuevas filas válidas para agregar.")
        return

    for columna in df_importacion.columns:
        if fc.es_columna_tipo_xxYY(columna) and columna not in df_maestro.columns:
            df_maestro[columna] = None

    df_nuevas_filas = pd.DataFrame(filas_a_agregar)
    df_maestro = pd.concat([df_maestro, df_nuevas_filas], ignore_index=True)

    df_maestro.to_excel(archivo_maestro, index=False)
    logger.info(f"Se han agregado {len(filas_a_agregar)} nuevas filas válidas al archivo maestro.")
    print(f"Se han agregado {len(filas_a_agregar)} nuevas filas válidas al archivo maestro.")

def run(tipo_PLC, codigo_columna_maestra):
    archivo_maestro = f".\\data\\1_hmi_master_translates_{tipo_PLC}.xlsx"
    archivo_importacion = fc.select_file("xlsx")
    if archivo_importacion:
        importar(tipo_PLC, codigo_columna_maestra, archivo_maestro, archivo_importacion)

if __name__ == "__main__":
    tipo_PLC = "siemens"
    codigo_columna_maestra = "it-IT"
    run(tipo_PLC, codigo_columna_maestra)